package collisionDetector.detector;


import collisionDetector.detector.interfaces.ICollidor;
import collisionDetector.detector.motionCreator.Motion;
import collisionDetector.main.data.Constants;
import collisionDetector.main.dataStructures.Collision;
import collisionDetector.main.dataStructures.HashMap;
import collisionDetector.main.dataStructures.Iterator;
import collisionDetector.main.dataStructures.LinkedList;
import collisionDetector.main.dataStructures.List;
import collisionDetector.main.dataStructures.Reducer_2;
import collisionDetector.main.dataStructures.Vector3d;





public class Collidor implements ICollidor{

	public List lookForCollisions(List motions) {

		final Reducer_2 r = new Reducer_2(Constants.GOOD_VOXEL_SIZE);

		System.out.println("Looking for collisions..");
		
	  final List check = reduceCollisionSet(r, motions);
    final CollisionCollector c = new CollisionCollector();
    for (final Iterator iter = check.iterator(); iter.hasNext();)
        c.addCollisions(determineCollisions((List) iter.next()));
    
    System.out.println("Collidor:" + c.getCollisions().size());
    return c.getCollisions();
		
		
	}

	/**
   * Takes a List of Motions and returns an List of Lists of Motions, where the inner lists implement RandomAccess.
   * Each Vector of Motions that is returned represents a set of Motions that might have collisions.
   */
  public List reduceCollisionSet(final Reducer_2 it, final List motions) {

      final HashMap voxel_map = new HashMap();
      final HashMap graph_colors = new HashMap();

      for (final Iterator iter = motions.iterator(); iter.hasNext();)
          it.performVoxelHashing((Motion) iter.next(), voxel_map, graph_colors);

      final List ret = new LinkedList();
      for (final Iterator iter = voxel_map.values().iterator(); iter.hasNext();) {
          final List cur_set = (List) iter.next();
          if (cur_set.size() > 1) ret.add(cur_set);
      }
      return ret;
  }
	

  private List determineCollisions(final List motions) {
    final List ret = new LinkedList();
    for (int i = 0; i < motions.size() - 1; i++) {
        final Motion one = (Motion) motions.get(i);
        for (int j = i + 1; j < motions.size(); j++) {
            final Motion two = (Motion) motions.get(j);
            final Vector3d vec = one.findIntersection(two);
            if (vec != null) ret.add(new Collision(one.getAircraft(), two.getAircraft(), vec));
        }
    }
    return ret;
  }
	

}
