package collisionDetector.main;

import javax.realtime.ImmortalMemory;
import javax.realtime.LTMemory;
import javax.realtime.MemoryArea;
import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.RealtimeThread;
import javax.realtime.RelativeTime;

import collisionDetector.detector.Collidor;
import collisionDetector.detector.Detector;
import collisionDetector.detector.motionCreator.MotionCreator;
import collisionDetector.detector.reciever.Receiver;
import collisionDetector.main.data.Constants;
import collisionDetector.main.interceptor.DetectorInterceptor;
import collisionDetector.main.interceptor.IStateTable;
import collisionDetector.main.interceptor.StateTableInterceptor;
import collisionDetector.main.util.ContextChecker;
import collisionDetector.simulator.MySimulator;
import collisionDetector.simulator.SerializedFrame;





public class CollisionDetector {

	
	ImmortalAllocation immAlloc;
	final int CONST_FRAMES = 100;
	
	MyRunner myRunner;
	
	IStateTable table = null;
	
	public void init(String[] args) {
		
	// instantiating the immortal component
		myRunner = allocInImmortal();
		
		
		
		table = immAlloc.table;
		
		SerializedFrame[] frames = new SerializedFrame[CONST_FRAMES];
		MySimulator.generate(args,frames,immAlloc.receiver);
	
		
		
		
		
	}
	
	public void start() {
		
		
		
		myRunner.start();
	}


	
	/**
	 * Ales
	 * 
	 * 
	 * 
	 * instantiating the immortal component - productionLine
	 * 
	 * 
	 * @return
	 */
	private MyRunner allocInImmortal() {
	
		
		
		
		// runnable thread containing the logic (instantiation proceedures) that has to be
		// executed in the immortal memory
		immAlloc = new ImmortalAllocation();
	
		//realtime thread working in immortal memory
		// - this thread do the instantiation - it will execute the immAlloc thread 
		RealtimeThread rt = new RealtimeThread(null, null, null, ImmortalMemory
				.instance(), // Start the thread in immortal
				null, immAlloc);
		// running the instantiation
		rt.start();
	
		//TODO: waiting until the instantiation will finish - here will be a problem on truly real-time machines
		try {
			RealtimeThread.sleep(2000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
	
			e.printStackTrace();
		}
		//System.out.println("Checking the immortal allocation result (should be non null):"+ immAlloc.productionLine);
	
		if (immAlloc.exception != null) {
			System.out.println("ERROR: exception thrown during intialization.");
			System.out.println("Exception:" + immAlloc.exception.toString() + "    "
					+ immAlloc.exception.getMessage());
			immAlloc.exception.printStackTrace();
	
			return null;
		}
	
		if (!(MemoryArea.getMemoryArea(immAlloc.runner) instanceof ImmortalMemory)) {
			System.out.println("ERROR: Runner is NOT in immortal.");
			System.out.println("Is in: "
					+ MemoryArea.getMemoryArea(immAlloc.runner));
			return immAlloc.runner;
		}
	
		System.out.println("Runner is immortal, instantiation... OK.");
		return immAlloc.runner;
	}



}




class ImmortalAllocation implements Runnable {

  public MyRunner runner = null;
  //public IReceiver iReceiver = null;
 
  public IStateTable table = null;
  
  public Detector detector;
  public MotionCreator motionCreator;
  public Receiver receiver;
  public Collidor collidor;
  
  
	public Exception exception = null;

	/**
	 * just some logic for printing on Console when being in immortal - since its not so trivial...
	 */
	final LTMemory scopedA = new LTMemory(8 * 1024);
	Runnable runTh = new Runnable() {
		public void run() {
			//System.out.println("Mem Consumed " + scopedA.memoryConsumed());
			//System.out.println("immortal memory consumed " + ImmortalMemory.instance().memoryConsumed()  );
			System.out.println("Mem Consumed " + scopedA.memoryConsumed());
			System.out.println("immortal memory consumed "
					+ ImmortalMemory.instance().memoryConsumed());
		}
	};

	private void memConsumed() {
		scopedA.enter(runTh);
	}

	public void run() {

		try {

			memConsumed();

		

			ContextChecker.checkContext(ContextChecker.IMMORTAL_CONTEXT);
			//ContextChecker.checkContext(ContextChecker.NHRT_THREAD);
			
			
		
	    
			runner = new MyRunner(new PriorityParameters(Constants.PRIORITY), new PeriodicParameters(null,// start
	        new RelativeTime(Constants.PERIOD, 0),// period
	        null,// new RelativeTime(5,0),//cost
	        null,// new RelativeTime(50,0),//deadline
	        null, null), ImmortalMemory.instance());
			
		  
		
			table = new StateTable();
			
			detector = new Detector(Constants.GOOD_VOXEL_SIZE);
	    detector.bindRunner(runner);
	    
	    DetectorInterceptor interceptor = new DetectorInterceptor(detector);
	    runner.iDetector = interceptor;
	    
	    receiver = new Receiver();
			motionCreator = new MotionCreator();
			collidor = new Collidor();
	    
			detector.bindReceiver(receiver);
			detector.bindMotionCreator(motionCreator);
			detector.bindCollidor(collidor);
			
		
	    
	    
			
			
			StateTableInterceptor tableInterceptor = new StateTableInterceptor(table);
			motionCreator.bindTable(tableInterceptor);
			

			
			
			//iReceiver = runner.detector.getReceiver();
			
		
			memConsumed();


		} catch (Exception e) {
			//throw new org.objectweb.fractal.julia.factory.ChainedInstantiationException(e,null,e.getClass().getName()+": "+e.getMessage());
			exception = e;

		}
	}

}



