package collisionDetector.main;

import javax.realtime.ImmortalMemory;
import javax.realtime.LTMemory;
import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.RealtimeThread;
import javax.realtime.RelativeTime;

import collisionDetector.detector.Collidor;
import collisionDetector.detector.Detector;
import collisionDetector.detector.motionCreator.MotionCreator;
import collisionDetector.detector.reciever.Receiver;
import collisionDetector.main.data.Constants;
import collisionDetector.main.interceptor.DetectorInterceptor;
import collisionDetector.main.interceptor.IStateTable;
import collisionDetector.main.interceptor.StateTableInterceptor;
import collisionDetector.main.util.ContextChecker;
import collisionDetector.simulator.MySimulator;
import collisionDetector.simulator.SerializedFrame;

public class MainBetterThanLife {

	
	final static int CONST_FRAMES = 100;
	
	
	/**
	 * @param args
	 */
	public static void main(final String[] args) {
		
		
		
		//variant1();
		
		variant2(args);

	}

	
//	public static void variant1() {
//		
//		Runnable logic = new Runnable() {
//			public void run() {
//				AllocationRunnable alloc = new AllocationRunnable();
//	  		
//	  		alloc.runner.start();
//	    }
//		};
//		
//		RealtimeThread rt = new RealtimeThread(null,null,null, ImmortalMemory.instance(),
//				null,logic);
//		rt.start();
//		
//	}
	
	
  public static void variant2(final String[] args) {
		
		AllocationRunnable alloc = new AllocationRunnable();
		
		RealtimeThread rt = new RealtimeThread(null,null,null, ImmortalMemory.instance(),
				null,alloc);
		rt.start();
		
		
		SerializedFrame[] frames = new SerializedFrame[CONST_FRAMES];
		MySimulator.generate(args,frames,alloc.receiver);
	
		
		
		
		alloc.runner.start();
		
	}
	
	
}


  




class AllocationRunnable implements Runnable {

  public MyRunner runner = null;
  //public IReceiver iReceiver = null;
 
  public IStateTable table = null;
  
  public Detector detector;
  public MotionCreator motionCreator;
  public Receiver receiver;
  public Collidor collidor;
  public DetectorInterceptor detectorInterceptor;
  
  
  
	public Exception exception = null;

	/**
	 * just some logic for printing on Console when being in immortal - since its not so trivial...
	 */
	final LTMemory scopedA = new LTMemory(8 * 1024);
	Runnable runTh = new Runnable() {
		public void run() {
			//System.out.println("Mem Consumed " + scopedA.memoryConsumed());
			//System.out.println("immortal memory consumed " + ImmortalMemory.instance().memoryConsumed()  );
			System.out.println("Mem Consumed " + scopedA.memoryConsumed());
			System.out.println("immortal memory consumed "
					+ ImmortalMemory.instance().memoryConsumed());
		}
	};

	private void memConsumed() {
		scopedA.enter(runTh);
	}

	public void run() {

		try {

			memConsumed();

		

			ContextChecker.checkContext(ContextChecker.IMMORTAL_CONTEXT);
			ContextChecker.checkContext(ContextChecker.RT_THREAD);
			
			
		
	    
			runner = new MyRunner(new PriorityParameters(Constants.PRIORITY), new PeriodicParameters(null,// start
	        new RelativeTime(Constants.PERIOD, 0),// period
	        null,// new RelativeTime(5,0),//cost
	        null,// new RelativeTime(50,0),//deadline
	        null, null), ImmortalMemory.instance());
			
		  
		
			table = new StateTable();
			
			detector = new Detector(Constants.GOOD_VOXEL_SIZE);
	    detector.bindRunner(runner);
	    
	    detectorInterceptor = new DetectorInterceptor(detector);
	    runner.iDetector = detectorInterceptor;
	    
	    receiver = new Receiver();
			motionCreator = new MotionCreator();
			collidor = new Collidor();
	    
			detector.bindReceiver(receiver);
			detector.bindMotionCreator(motionCreator);
			detector.bindCollidor(collidor);
			
		
	    
	    
			
			
			StateTableInterceptor tableInterceptor = new StateTableInterceptor(table);
			motionCreator.bindTable(tableInterceptor);
			

			
			
			//iReceiver = runner.detector.getReceiver();
			
		
			memConsumed();
			
			


		} catch (Exception e) {
			//throw new org.objectweb.fractal.julia.factory.ChainedInstantiationException(e,null,e.getClass().getName()+": "+e.getMessage());
			exception = e;

		}
	}

}
