package collisionDetector.simulator;

import collisionDetector.main.dataStructures.Vector3d;


/**
 * The interface <code>Region</code> represents a region within 2d or 3d space.
 * It has methods for checking whether a point is contained within the region
 * and also finding the distance to the region from some exterior point.
 * @author Ben L. Titzer
 **/
public interface Region {
    /**
     * The <code>contains</code> method determines whether a specified
     * point lies within the region.
     * @param point the point which might be contained in the region
     * @returns true if this region contains the point specified
     **/
    public boolean contains(Vector3d point);

    /**
     * The <code>distance</code> method determines the shortest distance
     * from the given point to the edge of the region. In the case where
     * the point is contained in the region, this method will return 0.
     * @param point the point to calculate the distance from
     * @returns the shortest straight line distance from this point to
     * the edge of the region, 0 if the point is contained in
     * the region
     **/
    public float distance(Vector3d point);
}
