package collisionDetector.simulator;

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.StringTokenizer;

import collisionDetector.simulator.data.SimulatorConstants;

class SimMediator {
    public static final FrameFactory frameFactory = new FrameFactory();
    public static final int DEFAULT_RESOLUTION = 600;
    public static final float MIN_Y = 0.0f;
    public static final float MIN_X = 0.0f;
    public static final float MAX_Y = 1000.0f;
    public static final float MAX_X = 1000.0f;
    public static final int TIME_SCALE = 10;
    public static final int FPS = 60;
    public static final int MAX_FRAMES = 30000;

    public MySimulator buildSimulator(final String s) {
        final SimAircraftProducer prod = new SimAircraftProducer();
        final float param_min_x = MIN_X;
        final float param_min_y = MIN_Y;
        final float param_max_x = MAX_X;
        final float param_max_y = MAX_Y;
        final MySimulator.Parameters params = new MySimulator.Parameters();
        params.fps = FPS;
        params.max_craft = SimulatorConstants.MAX_CRAFT;
        params.timescale = TIME_SCALE;
        params.max_frames = MAX_FRAMES;

        MasterAirportCollection.readAirportAndNoFlyConfig("sim.conf");
        createSimulation(s, prod);

        final SimulationRegion region = new SimulationRegion(new RectangularRegion(param_min_x, param_min_y,
                param_max_x, param_max_y), null, null);
        params.director = prod;
        params.producer = prod;
        params.region = region;
        return new MySimulator(params);
    }

    //@SuppressWarnings("deprecation")
    protected static void createSimulation(final String filename, final SimAircraftProducer prod) {
        DataInputStream fin = null;
        try {
            fin = new DataInputStream(new FileInputStream(filename));
        } catch (final FileNotFoundException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        while (true) {
            String line = null;
            try {
                line = fin.readLine();
            } catch (final IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            if (line == null) break;
            final StringTokenizer tox = new StringTokenizer(line, "\t");
            if (!tox.hasMoreElements()) break;
            final String command = tox.nextToken();
            final String name = tox.nextToken();
            final String[] param = new String[] { tox.nextToken(), tox.nextToken(), tox.nextToken() };
            try {
                prod.addAircraft(name, param, MasterAirportCollection.lookup(tox.nextToken()), MasterAirportCollection
                        .lookup(tox.nextToken()));
            } catch (final Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
}
